package arzumify.coretechs.energy;

import arzumify.coretechs.BlockEntityTypes;
import arzumify.polyenergy.api.EnergyProvider;
import arzumify.polyenergy.api.EnergyReceiver;
import arzumify.polyenergy.impl.MatchMaker;
import arzumify.located.math.Vec3i;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerBlockEntityEvents;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class VoltmeterEntity extends BlockEntity implements EnergyReceiver, ServerBlockEntityEvents.Unload, ServerBlockEntityEvents.Load {
    private final Set<EnergyProvider> providers = ConcurrentHashMap.newKeySet();

    public VoltmeterEntity(BlockPos pos, BlockState state) {
        super(BlockEntityTypes.VOLTMETER, pos, state);
        ServerBlockEntityEvents.BLOCK_ENTITY_UNLOAD.register(this);
        ServerBlockEntityEvents.BLOCK_ENTITY_LOAD.register(this);
    }

    @Override
    public void ready(EnergyProvider energyProvider) {
        System.out.println("Ready " + energyProvider);
        providers.add(energyProvider);
    }

    @Override
    public void unready(EnergyProvider energyProvider) {
        System.out.println("Unready " + energyProvider);
        providers.remove(energyProvider);
    }

    public static void tick(World ignoredWorld, BlockPos ignoredPos, BlockState ignoredState, VoltmeterEntity voltmeter) {
        for (EnergyProvider provider : voltmeter.providers) {
            System.out.println("Extracted " + provider.extract(Long.MAX_VALUE, voltmeter));
        }
    }

    @Override
    public void onLoad(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Add(this);
        reload();
    }

    @Override
    public void onUnload(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Remove(this);
        for (EnergyProvider provider : providers) {
            provider.unready(this);
        }
    }

    @Override
    public Collection<Vec3i> points() {
        return List.of(new Vec3i(pos.getX(), pos.getY(), pos.getZ()));
    }

    @Override
    public String dimension() {
        return Objects.requireNonNull(this.getWorld()).getRegistryKey().getValue().toString();
    }

    public void reload() {
        var found = MatchMaker.matchMaker.Search(this);
        if (found != null) {
            for (var foundEnergyBlocks : found) {
                if (foundEnergyBlocks instanceof EnergyProvider && !providers.contains(foundEnergyBlocks)) {
                    ((EnergyProvider) foundEnergyBlocks).ready(this);
                }
            }
        }
    }
}
